
-- 2008-11-07	We are trying to get rid of the old Piggybank_money table since handling
--				the per-player settings ourselves is needless hassle.
--				The amount is now saved in PiggybankPrefs.money. For a transition period, we will
--				take over the money value from the old table.
--				All code involved is marked "for compatability only" and may be removed
--				in some future version.
-- 2008-11-07	New saved variable PiggybankPrefs (table).
--				Currently stored variables:
--				- position (table)
--				We use this to move the Piggybank frame to its old position even if the addon
--				had been switched off or uninstalled in the meantime.
--				- fadeOut (bool)
--				If true, we decrease opacity on mouse leave.
-- 2008-11-09	Added min and max settings for backpack.
--				At startup we compare backpack gold to these values and move money to or from
--				the piggy bank accordingly.
-- 				Added standard settings page for fade out and backpack min/max settings.
--				Added localization for settings.
-- 2008-11-17	Removed compatibility code.
--				Added live tracking; money in piggybank will be adjusted according to set limits
--				whenever the player gains or loses money.
--				Color coded money display: 	white = manual mode
--											yellow = tracking mode
--											orange = manual override (*)
--				(* I.e. the player manually adjusted the amount in the piggy bank while in
--				tracking mode. The player gaining or losing money will resume tracking.)
--				Added key bindings. 		
--					Toggle Tracking: Switch live tracking on/off
--					Sync Money: Adjust amount in piggy bank now
--					Swap Money: Swap amount of money in piggy bank with amount in backpack
-- 2010-11-03	Updated for 4.x


-- Begin Localization

if (GetLocale() == "deDE") then

Piggybank_Loc = {
	FadeOut = "Ausblenden, wenn Maus das Fenster verläßt",
	TrackMoney = "Überwache Geldbewegungen",
	BackpackMin = "Mindestens im Rucksack:",
	BackpackMax = "Höchstens im Rucksack:",
	BindingsHeader = "Piggybank",
	BindingToggleTracking = "Überwachung ein/aus",
	BindingSync = "Betrag abgleichen",
	BindingSwap = "Tausche Betrag"
}

else

Piggybank_Loc = {
	FadeOut = "Fade Out on Mouse Leave",
	TrackMoney = "Track Changes",
	BackpackMin = "Min. Amount in Backpack:",
	BackpackMax = "Max. Amount in Backpack:",
	BindingsHeader = "Piggybank",
	BindingToggleTracking = "Toggle Tracking",
	BindingSync = "Sync Money",
	BindingSwap = "Swap Money"
}

end


-- End Localization


PiggybankPrefs = {}
PiggybankPrefs.money = -1
PiggybankPrefs.fadeOut = false
PiggybankPrefs.trackMoney = false
PiggybankPrefs.backpackMin = 0
PiggybankPrefs.backpackMax = 0

local Piggybank_OriginalGetMoney = nil
local Piggybank_OriginalBuyMerchantItem = nil
local Piggybank_OriginalPickupMerchantItem = nil

local Playername = nil
local Realmname = GetCVar("realmName")

local isFadedOut = false
local variablesLoaded = false
local loggedIn = false
local manualOverride = false

function Piggybank_Print(text)
	DEFAULT_CHAT_FRAME:AddMessage("Piggybank: "..text, 1.0, 0.0, 0.0)
end

function Piggybank_MoneyChanged()
	local oldval = PiggybankPrefs.money;
	local newval = MoneyInputFrame_GetCopper(PiggybankMoneyFrame)
	if oldval ~= newval then
		if newval > Piggybank_OriginalGetMoney() then
			-- wrong input: reset
			newval = Piggybank_OriginalGetMoney()
			manualOverride = false
			if PiggybankPrefs.trackMoney then
				newval = Piggybank_Autoadjust()
			end
		else
			if PiggybankPrefs.trackMoney then
				-- manually adjusted amount in piggybank
				manualOverride = true
			end
		end
	end
	PiggybankPrefs_Money_Set(newval)
end

function Piggybank_GetMoney()
	local realmoney = Piggybank_OriginalGetMoney()

	local result = PiggybankPrefs.money
	
	if result < 0 then -- error
		result = realmoney
	else
		if realmoney < result then -- too much in piggy bank
			PiggybankPrefs_Money_Set(realmoney)
			result = 0 -- we're broke
		else
			result = realmoney - PiggybankPrefs.money
		end
	end	
	return result
end

function Piggybank_BuyMerchantItem(index, qty)
	local name, texture, price, quantity, numAvailable, isUsable = GetMerchantItemInfo(self:GetID())

	if (price > GetMoney()) then
		UIErrorsFrame:AddMessage("You can't afford that", 1.0, 0.0, 0.0, 1.0, UIERRORS_HOLD_TIME)
	else
		Piggybank_OriginalBuyMerchantItem(index, qty)
	end
end

function Piggybank_PickupMerchantItem(index)
	local name, texture, price, quantity, numAvailable, isUsable = GetMerchantItemInfo(self:GetID())

	if (price > GetMoney()) then
		UIErrorsFrame:AddMessage("You can't afford that", 1.0, 0.0, 0.0, 1.0, UIERRORS_HOLD_TIME)
	else
		Piggybank_OriginalPickupMerchantItem(index, qty)
	end
end

function Piggybank_SaveFramePosition()
	PiggybankPrefs.position = {}
	PiggybankPrefs.position.point, PiggybankPrefs.position.relativeTo, PiggybankPrefs.position.relativePoint, PiggybankPrefs.position.xOffset, PiggybankPrefs.position.yOffset = PiggybankFrame:GetPoint()
end

function Piggybank_RestoreFramePosition()
	PiggybankFrame:ClearAllPoints()
	local pos = PiggybankPrefs.position
	if pos ~= nil then
		PiggybankFrame:SetPoint(pos.point, pos.relativeTo, pos.relativePoint, pos.xOffset, pos.yOffset)
	end
end

function Piggybank_UpdateMoneyFrame()
	MoneyInputFrame_SetCopper(PiggybankMoneyFrame, PiggybankPrefs.money)
	local r, g, b
	if PiggybankPrefs.trackMoney then
		if manualOverride then
			-- manual override
			r = 1.0; g = 0.5; b = 0.0
		else
			-- automatically managed
			r = 1.0; g = 1.0; b = 0.0
		end
	else
		-- manually set
		r = 1.0; g = 1.0; b = 1.0
	end
	MoneyInputFrame_SetTextColor(PiggybankMoneyFrame, r, g, b)
end


function Piggybank_Autoadjust()
	local result = PiggybankPrefs.money
	manualOverride = false
	if PiggybankPrefs.backpackMax > 0 then
		local allMoney = Piggybank_OriginalGetMoney()
		local backpackMoney = Piggybank_GetMoney()
		if backpackMoney > PiggybankPrefs.backpackMax then
			if PiggybankPrefs.backpackMin > 0 then
				result = max(0, allMoney - PiggybankPrefs.backpackMin)
			else
				result = max(0, allMoney - PiggybankPrefs.backpackMax)
			end
		elseif backpackMoney < PiggybankPrefs.backpackMin then
			result = max(0, allMoney - PiggybankPrefs.backpackMin)
		end
		PiggybankPrefs_Money_Set(result)
	end
	return result
end

function Piggybank_SwapMoney()
	-- swap amount of money in piggybank with amount in backpack
	local newval = Piggybank_OriginalGetMoney()-PiggybankPrefs.money
	manualOverride = true
	PiggybankPrefs_Money_Set(newval)
end

function Piggybank_ToggleTracking()
	PiggybankPrefs_TrackMoney_Set(not PiggybankPrefs.trackMoney)
end

function PiggyBank_VariablesLoaded(self)
	-- set default prefs values if nothing stored
	if PiggybankPrefs.fadeOut == nil then
		PiggybankPrefs.fadeOut = false
	end
	if PiggybankPrefs.trackMoney == nil then
		PiggybankPrefs.trackMoney = false
	end
	if PiggybankPrefs.backpackMin == nil then
		PiggybankPrefs.backpackMin = 0
	end
	if PiggybankPrefs.backpackMax == nil then
		PiggybankPrefs.backpackMax = 0
	end

	if PiggybankPrefs.trackMoney then
		self:RegisterEvent("PLAYER_MONEY")
	else
		self:UnregisterEvent("PLAYER_MONEY")
	end

	Piggybank_Localize()
	Piggybank_SetupUI()
	Piggybank_RestoreFramePosition()
	PiggybankSettings_Localize()
	PiggybankSettings_Update()
	Piggybank_Autoadjust()
	Piggybank_FadeOut()
end

-- prefs setters

function PiggybankPrefs_Money_Set(amount)
	if PiggybankPrefs.money ~= amount then
		PiggybankPrefs.money = amount
		
		-- update backpack money frame
		local Backpack_MoneyFrame = getglobal("ContainerFrame1MoneyFrame")
		MoneyFrame_UpdateMoney(Backpack_MoneyFrame)
	end
	Piggybank_UpdateMoneyFrame()
end

function PiggybankPrefs_FadeOut_Set(flag)
	if PiggybankPrefs.fadeOut ~= flag then
		PiggybankPrefs.fadeOut = flag
		if PiggybankPrefs.fadeOut then
			Piggybank_FadeOut()
		else
			Piggybank_FadeIn()
		end
	end
end

function PiggybankPrefs_TrackMoney_Set(flag)
	if PiggybankPrefs.trackMoney ~= flag then
		PiggybankPrefs.trackMoney = flag
		if PiggybankPrefs.trackMoney then
			PiggybankFrame:RegisterEvent("PLAYER_MONEY")
			Piggybank_Autoadjust()
		else
			PiggybankFrame:UnregisterEvent("PLAYER_MONEY")
			Piggybank_UpdateMoneyFrame()
		end
	end
end

function PiggybankPrefs_BackpackMin_Set(amount)
	if PiggybankPrefs.backpackMin ~= amount then
		PiggybankPrefs.backpackMin = amount
	end
end

function PiggybankPrefs_BackpackMax_Set(amount)
	if PiggybankPrefs.backpackMax ~= amount then
		PiggybankPrefs.backpackMax = amount
	end
end


-- animation

function Piggybank_FadeInValue(self, fraction)
	return 0.2+(0.8*fraction)
end

function Piggybank_FadeOut()
	if PiggybankPrefs.fadeOut then
		if not isFadedOut then
			isFadedOut = true
			animTable = { totalTime = 0.5, updateFunc = PiggybankFrame.SetAlpha, getPosFunc = Piggybank_FadeInValue }
			SetUpAnimation(PiggybankFrame, animTable, nil, true)
		end
	end
end

function Piggybank_FadeIn()
	if isFadedOut then
		isFadedOut = false
		animTable = { totalTime = 0.2, updateFunc = PiggybankFrame.SetAlpha, getPosFunc = Piggybank_FadeInValue }
		SetUpAnimation(PiggybankFrame, animTable, nil, false)
	end
end

function Piggybank_SetupUI()
	-- adjust frame and backdrop texture and color
	PiggybankFrame:SetBackdrop({bgFile = "Interface\\CharacterFrame\\UI-Party-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
	PiggybankFrame:SetBackdropBorderColor(.5,.5,.5,1)
end

function Piggybank_Localize()
	BINDING_HEADER_PIGGYBANK_HEADER = Piggybank_Loc.BindingsHeader
	BINDING_NAME_PIGGYBANK_TOGGLETRACKING = Piggybank_Loc.BindingToggleTracking
	BINDING_NAME_PIGGYBANK_SYNC = Piggybank_Loc.BindingSync
	BINDING_NAME_PIGGYBANK_SWAP = Piggybank_Loc.BindingSwap
end

-- event handling

function Piggybank_OnLoad(self)
	Piggybank_OriginalGetMoney = GetMoney
	GetMoney = Piggybank_GetMoney
	Piggybank_OriginalBuyMerchantItem = BuyMerchantItem
	BuyMerchantItem = Piggybank_BuyMerchantItem
	Piggybank_OriginalPickupMerchantItem = PickupMerchantItem
	PickupMerchantItem = Piggybank_PickupMerchantItem
	if GetMoney == Piggybank_OriginalGetMoney then
		Piggybank_Print("GetMoney() hook failed")
	end

	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("PLAYER_LOGIN")

end

function Piggybank_OnEvent(self, event, ...)
	if event == "PLAYER_LOGIN" then
		loggedIn = true
		self:UnregisterEvent("PLAYER_LOGIN")
	end
	if event == "VARIABLES_LOADED" then
		variablesLoaded = true
		self:UnregisterEvent("VARIABLES_LOADED")
	end
	if event == "PLAYER_MONEY" then
		Piggybank_Autoadjust()
	end
	if variablesLoaded and loggedIn then
		Playername = UnitName("player")
		PiggyBank_VariablesLoaded(self)
	end
end

function Piggybank_OnEnter()
	--[[ test: move to the left - hey, that actually works! :)
	local point, relativeTo, relativePoint, xOffset, yOffset = PiggybankFrame:GetPoint()
	xOffset = xOffset - 20
	PiggybankFrame:ClearAllPoints()
	PiggybankFrame:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset)
	--]]
	
	Piggybank_FadeIn()
end

function Piggybank_OnLeave()
	Piggybank_SaveFramePosition()
	if not MouseIsOver(PiggybankFrame) then
		Piggybank_FadeOut()
	end
end


-- Settings Panel

function PiggybankSettings_OnLoad(self)
	local settingsPanel = self
	settingsPanel.name = "Piggybank"
	settingsPanel.okay = function (self) PiggybankSettings_Okay(); end;
	settingsPanel.default = function (self) PiggybankSettings_Default(); PiggybankSettings_Update(); end;
	InterfaceOptions_AddCategory(settingsPanel)
end

function PiggybankSettings_Localize()

	PiggybankSettings_FadeOutText:SetText(Piggybank_Loc["FadeOut"])
	PiggybankSettings_TrackMoneyText:SetText(Piggybank_Loc["TrackMoney"])
	PiggybankSettings_BackpackMinLabel:SetText(Piggybank_Loc["BackpackMin"])
	PiggybankSettings_BackpackMaxLabel:SetText(Piggybank_Loc["BackpackMax"])
end

function PiggybankSettings_Okay()
	PiggybankPrefs_FadeOut_Set(PiggybankSettings_FadeOut:GetChecked())
	PiggybankPrefs_TrackMoney_Set(PiggybankSettings_TrackMoney:GetChecked())
	local oldBackpackMin = PiggybankPrefs.backpackMin
	local oldBackpackMax = PiggybankPrefs.backpackMax
	PiggybankPrefs_BackpackMin_Set(MoneyInputFrame_GetCopper(PiggybankSettings_BackpackMin))
	PiggybankPrefs_BackpackMax_Set(MoneyInputFrame_GetCopper(PiggybankSettings_BackpackMax))
	if (oldBackpackMin ~= PiggybankPrefs.backpackMin) or (oldBackpackMax ~= PiggybankPrefs.backpackMax) then
		Piggybank_Autoadjust()
	end
end

function PiggybankSettings_Default()
	PiggybankPrefs.version = GetAddOnMetadata("Piggybank", "Version")
	PiggybankPrefs.fadeOut = false
	PiggybankPrefs.trackMoney = false
	PiggybankPrefs.backpackMin = 0
	PiggybankPrefs.backpackMax = 0
end

function PiggybankSettings_Update()
	if PiggybankSettings_TrackMoney then
		PiggybankSettings_FadeOut:SetChecked(PiggybankPrefs.fadeOut)
		PiggybankSettings_TrackMoney:SetChecked(PiggybankPrefs.trackMoney)
		-- for some reason we need to clear the input fields first so that they get displayed
		MoneyInputFrame_SetCopper(PiggybankSettings_BackpackMin, 0)
		MoneyInputFrame_SetCopper(PiggybankSettings_BackpackMax, 0)
		MoneyInputFrame_SetCopper(PiggybankSettings_BackpackMin, PiggybankPrefs.backpackMin)
		MoneyInputFrame_SetCopper(PiggybankSettings_BackpackMax, PiggybankPrefs.backpackMax)
	else
		-- still not sure why this might happen
--		Piggybank_Print("UI not loaded yet")
	end
end

function PiggybankSettings_VerifyBackpackMin()
	local backpackMin = MoneyInputFrame_GetCopper(PiggybankSettings_BackpackMin)
	local backpackMax = MoneyInputFrame_GetCopper(PiggybankSettings_BackpackMax)
	if (backpackMin > backpackMax) then
		MoneyInputFrame_SetTextColor(PiggybankSettings_BackpackMin, 1, 0, 0)
	else
		MoneyInputFrame_SetTextColor(PiggybankSettings_BackpackMin, 1, 1, 1)
		MoneyInputFrame_SetTextColor(PiggybankSettings_BackpackMax, 1, 1, 1)
	end
end

function PiggybankSettings_VerifyBackpackMax()
	local backpackMin = MoneyInputFrame_GetCopper(PiggybankSettings_BackpackMin)
	local backpackMax = MoneyInputFrame_GetCopper(PiggybankSettings_BackpackMax)
	if (backpackMin > backpackMax) then
		MoneyInputFrame_SetTextColor(PiggybankSettings_BackpackMax, 1, 0, 0)
	else
		MoneyInputFrame_SetTextColor(PiggybankSettings_BackpackMin, 1, 1, 1)
		MoneyInputFrame_SetTextColor(PiggybankSettings_BackpackMax, 1, 1, 1)
	end
end

